
;*********************************************************************
;  Debugausgabe
;*********************************************************************

#ifdef	USE_DEBUG

;************************** UART Routinen *************************

;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash ber die serielle Schnittstelle
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_debug_send_text:
		push	ZH
		push	ZL
		push	TEMP_0

uart_debug_send_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	uart_debug_send_text_end					; Ende

        rcall   uart_debug_send_byte

		rjmp    uart_debug_send_text_loop					; weiter

uart_debug_send_text_end:
		pop		TEMP_0
 		pop		ZL
		pop	ZH

        ret

;******************************* Sub-Debug *********************************
; Sub: Sendet NL/CR ber die serielle Schnittstelle
; Parameter:	-
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_debug_send_newline:
        push    TEMP_0

		ldi     TEMP_0,$0A
        rcall   uart_debug_send_byte
		ldi     TEMP_0,$0D
        rcall   uart_debug_send_byte

        pop     TEMP_0

		ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als HEX-Dump ber UART
; Parameter: 	TEMP_0
; Return:      	-
; Scratch-Reg:	-
;***************************************************************************

uart_debug_send_byte_hex:
        push TEMP_0
		push TEMP_1

        ldi  TEMP_1,7
        push TEMP_0
		swap TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30								; nach Ziffer
		cpi  TEMP_0,$3A								; 0...9
    	brlo uart_debug_send_byte_hex_1

        add  TEMP_0,TEMP_1                 			; A...F

uart_debug_send_byte_hex_1:
        rcall uart_debug_send_byte
		pop  TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                   			; nach Ziffer
		cpi  TEMP_0,$3A                   			; 0...9
		brlo uart_debug_send_byte_hex_2

        add  TEMP_0,TEMP_1                			; A...F

uart_debug_send_byte_hex_2:
        rcall uart_debug_send_byte

        pop   TEMP_1
		pop   TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:	TEMP_0 Datenbyte
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_debug_send_byte:

		rcall	uart_soft_send

        ret


;******************************** Sub-Intern *******************************
; Sub: Speicher als 32-Bit (4-Byte) Tabelle ausgeben
; Parameter:    Z - Adresse
; Parameter:    Y - Lnge
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

uart_debug_send_hex_tab:       
		ldi		TEMP_0,4

uart_debug_send_hex_tab_loop:
		lsr     YH                   				; /16
        ror     YL
		dec		TEMP_0
		brne	uart_debug_send_hex_tab_loop

uart_debug_send_hex_tab_line_loop:
        mov		TEMP_0,ZH            				; Adresse senden
        rcall	uart_debug_send_byte_hex
        mov     TEMP_0,ZL
        rcall   uart_debug_send_byte_hex

        ldi     TEMP_1,16							; Bytecounter

uart_debug_send_hex_tab_byte_loop:
        ldi     TEMP_0,' '           				; Leerzeichen
        rcall   uart_debug_send_byte

        ld      TEMP_0,Z+
        rcall   uart_debug_send_byte_hex

        dec		TEMP_1
        brne    uart_debug_send_hex_tab_byte_loop

        rcall   uart_debug_send_newline
        sbiw    Y,1
        brne    uart_debug_send_hex_tab_line_loop

        ret

;******************************** Sub-Intern *******************************
; Sub: Speicher als 32-Bit (4-Byte) Tabelle ausgeben
; Parameter:    Z - Adresse
; Parameter:    Y - Lnge
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

debug_flash:
		rcall	flash_read_id
;		rcall	flash_set_pagesize
;		rcall	flash_read_status
;		rcall	flash_read_security

		load_p	Z,2							; erster Sektor
		cbr		FLAGS_IRQ,(1<<FLAG_BUFFER)	; Buffer 1
		rcall	flash_read_page2buffer		; in Buffer einlesen
		rcall	flash_read_status			; auf Flash Ready warten

		rcall	flash_read_buffer2ram		; ins Ram kopieren
		
		rcall	uart_debug_send_newline
		load_p	Z,FLASH_BUF					; Adresse
		load_p	Y,256						; Lnge	

		rcall	uart_debug_send_hex_tab
		rcall	uart_debug_send_newline

		load_p	Y,FLASH_BUF
		load_p	Z,(2*dummy_daten)			; erster Sektor
		clr		TEMP_1

test_loop:
		lpm		TEMP_0,Z+
		st		Y+,TEMP_1
		dec		TEMP_1
		brne	test_loop

		rcall	flash_write_ram2buffer

		load_p	Z,2							; erster Sektor
		rcall	flash_write_buffer2page

		load_p	Z,2							; erster Sektor
		rcall	flash_read_page2buffer		; in Buffer einlesen
		rcall	flash_read_status		; auf Flash Ready warten

		rcall	flash_read_buffer2ram		; ins Ram kopieren
		
		rcall	uart_debug_send_newline
		load_p	Z,FLASH_BUF					; Adresse
		load_p	Y,256						; Lnge	
		rcall	uart_debug_send_hex_tab
		rcall	uart_debug_send_newline

		ret

;******************************** Sub-Intern *******************************
; Sub: Flash-Manufakturer und Device-ID anzeigen
; Parameter:    -
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

debug_show_flash_id:
		rcall	uart_debug_send_newline

		lds		TEMP_0,FLASH_MAN
		rcall	uart_debug_send_byte_hex

		ldi		TEMP_0,' '
		rcall	uart_debug_send_byte

		lds		TEMP_0,FLASH_DEV
		rcall	uart_debug_send_byte_hex

		ldi		TEMP_0,' '
		rcall	uart_debug_send_byte

		lds		TEMP_0,FLASH_DEV+1
		rcall	uart_debug_send_byte_hex

		rcall	uart_debug_send_newline

		ret

;******************************** Sub-Intern *******************************
; Sub: Flash-Status ausgeben
; Parameter:    -
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

debug_show_flash_status:
		rcall	uart_debug_send_byte_hex
		ldi		TEMP_0,' '
		rcall	uart_debug_send_byte

		ret

;******************************** Sub-Intern *******************************
; Sub: Reset-Text ausgeben
; Parameter:    -
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

debug_show_reset:
		rcall	uart_debug_send_newline
		rcall	uart_debug_send_newline
		load_p	Z,(2*text_reset)
		rcall	uart_debug_send_text
		rcall	uart_debug_send_newline

;		rcall	debug_flash
		load_p	Z,TITEL_BUF
		load_p	Y,128
		rcall	uart_debug_send_hex_tab
		rcall	uart_debug_send_newline
				
		ret	

#endif

#ifdef USE_DEBUG_LED

;******************************** Sub-Intern *******************************
; Sub: LED Anzeige
; Parameter:    -
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

debug_show_led_status:
		sbrs	FLAGS,FLAG_SOUND
		sbi		LED_PORT,LED_GREEN
		sbrc	FLAGS,FLAG_SOUND
		cbi		LED_PORT,LED_GREEN

		sbrs	FLAGS,FLAG_VOLUME
		sbi		LED_PORT,LED_YELLOW
		sbrc	FLAGS,FLAG_VOLUME
		cbi		LED_PORT,LED_YELLOW

		ret

#endif
