
; Definitionen fr Mega16

; Belegungen der Atmel-Ports:

;*****************************************************************

;*****************************************************************

;	PA0		ADC0		A0				Adressen Low U880		
;   PA1		ADC1		A1
;	PA2		ADC2		A2      
;   PA3		ADC3      	A3
;   PA4		ADC4		A4
;   PA5		ADC5		A5
;	PA6		ADC6		A6
;	PA7		ADC7		A7

;	PB0		XCK/T0		D0				Daten U880
;   PB1		T1			D1
;   PB2		INT2/AIN0   D2
;   PB3		OC0/AIN1    D3
;   PB4		SS			D4
;   PB5		MOSI		D5
;	PB6		MISO		D6
;	PB7		SCK			D7
				
;   PC0		SCL			A8				Adressen High U880
;   PC1		SDA			A9
;   PC2		TCK			A10
;   PC3		IMS			A11
;	PC4		TDO			A12
;	PC5		TDI			A13
;	PC6		TOSC1		A14
;	PC7		TOSC2		A15

;   PD0		RxD			RS232
;   PD1		TxD			RS232
;   PD2		INT0		/RD				/RD U880
;   PD3		INT1		/MREQ			/MREQ U880
;   PD4	    OC1B		/IORQ			/IORQ U880 
;   PD5		OC1A		/M1				/M1 U880
;   PD6		ICP1		/WR				/WR U880
;   PD7		OC2			/BUSRQ			/BUSRQ U880

;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 4400000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ    UART_BAUD_RATE  = 19200
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1

.equ	SOH				= 0x01								; XModem Start Of Header
.equ	ACK				= 0x06
.equ	NAK				= 0x15
.equ	EOT				= 0x04

.equ	UART_RETRY		= 5									; Anzahl Wiederholungen bei Blockfehler

.equ	RETURN			= 13
			
;**************************** Port-Bits ***************************

.equ	ADR_L_PORT		= PORTA
.equ	ADR_L_DDR		= DDRA

.equ	ADR_H_PORT		= PORTC
.equ	ADR_H_DDR		= DDRC

.equ	DATA_PORT		= PORTB
.equ	DATA_PIN		= PINB
.equ	DATA_DDR		= DDRB

.equ	CTRL_PORT		= PORTD
.equ	CTRL_DDR		= DDRD

.equ	Z80_RD			= PD2
.equ	Z80_MREQ		= PD3	
.equ	Z80_IORQ		= PD4	
.equ	Z80_M1			= PD5	
.equ	Z80_WR			= PD6
.equ	Z80_BUSRQ		= PD7	
	
;***************************** FLAGS ******************************

.equ	FLAG_STR_OK		= 0						; gltiger String empfangen
.equ	FLAG_STR_LEN	= 1						; String mehr als 16 Zeichen

.equ	FLAG_CMD_ERR	= 2						; unbekanntes Kommando
.equ	FLAG_PAR_ERR	= 3						; ungoltiger Parameter

;**************************** Ram-Reservierungen ******************

;--- Internes Ram ------------------------------------------------

.DSEG

.ORG    0x060

UART_BUF:	.byte		20

CMD_ADR:	.byte		 2

MEM_ANF:	.byte		 2
MEM_END:	.byte		 2

ADR_NIBBLE:	.byte		 4

MEM_BUF:	.byte	   1024
		
;**************************** Register ****************************

;--- lower Register -----------------------------------------------

.def	NULL		= r2

;--- higher Register ----------------------------------------------

.def	FLAGS		= r16					; Flags fr Funktionen

.def	BLOCK_CNT	= r17					; XModem-Blockzhler		
.def	BYTE_CNT	= r18					; Byte-Zhler XModem
.def	CHECKSUM	= r19					; XModem

.def	TEMP_2		= r23                   ; Temp-Register
.def	TEMP_1		= r24                   ; Temp-Register
.def	TEMP_0		= r25					; Temp-Register

; r26/r27, r28/r29, r30/r31 fr X,Y,Z reserviert 

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low

        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)

.endmacro

