
;**************************** Allgemeine Routinen **************************

;******************************* Subroutinen *******************************
; Sub: Zerlegt den String in UART_BUF
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

parse_string:
		load_p	Y,UART_BUF

		rcall	find_cmd			; Kommando suchen, Funktionsadresse nach CMD_ADR
		sbrc	FLAGS,FLAG_CMD_ERR
		rjmp	parse_string_end

		lds		ZL,CMD_ADR+0	
		lds		ZH,CMD_ADR+1
		icall	

parse_string_end:
		ret

;******************************* Subroutinen *******************************
; Sub: Dummy-Command fr unbekannten Buchstaben
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************
dummy:
		load_p	Z,text_cmd_err
		rcall	uart_send_rom_text
		rcall	uart_send_newline

		ret

;******************************* Subroutinen *******************************
; Sub: Holt Funktionsadresse nach CMD_ADR, Y zeigt auf Bufferposition
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

find_cmd:
		cbr		FLAGS,(1<<FLAG_CMD_ERR)

find_cmd_loop:
		ld		TEMP_0,Y+			; Byte holen
		tst		TEMP_0				; Ende?
		breq	find_cmd_err		; ja

		cpi		TEMP_0,' '			; Space?
		breq	find_cmd_loop		; weitersuchen

		cpi		TEMP_0,'a'
		brlo	find_cmd_err		; kein Kommando

		cpi		TEMP_0,'z'+1
		brsh	find_cmd_err

		load_p	Z,tab_command
		subi	TEMP_0,'a'
		add		ZL,TEMP_0
		adc		ZH,NULL				; zeiger auf Kommandotabelle
		lsl		ZL
		rol		ZH

		lpm		TEMP_0,Z+
		sts		CMD_ADR+0,TEMP_0
		lpm		TEMP_0,Z
		sts		CMD_ADR+1,TEMP_0

		rjmp	find_cmd_end

find_cmd_err:
		sbr		FLAGS,(1<<FLAG_CMD_ERR)

find_cmd_end:
		ret

;******************************* Subroutinen *******************************
; Sub: Holt Adresse nach X, Y zeigt auf Bufferposition
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

find_adr:
		cbr		FLAGS,(1<<FLAG_PAR_ERR)

		load_p	Z,ADR_NIBBLE
		ldi		TEMP_2,4			; Anzahl Nibble

find_adr_loop:
		ld		TEMP_0,Y+			; Byte holen
		tst		TEMP_0				; Ende?
		breq	find_adr_err		; ja

		cpi		TEMP_0,' '			; Space?
		breq	find_adr_loop		; weitersuchen

		sbiw	Y,1					; Korrektur Zeiger auf erstes nicht-Space

find_adr_ok:
		ld		TEMP_0,Y+			; Byte holen
		tst		TEMP_0				; Ende?
		breq	find_adr_err		; ja

		cpi		TEMP_0,'0'			; Sonderzeichen, ungltig
		brlo	find_adr_err

		cpi		TEMP_0,'9'+1
		brlo	find_adr_zif		; 0...9

		andi	TEMP_0,0b11011111	; in Grobuchstaben
		cpi		TEMP_0,'A'
		brlo	find_adr_err		; Fehler

		cpi		TEMP_0,'F'+1
		brsh	find_adr_err		; grer F

		subi	TEMP_0,7			; nach 10...15

find_adr_zif:
		subi	TEMP_0,'0'
		st		Z+,TEMP_0			; merken
		dec		TEMP_2
		brne	find_adr_ok

		ld		TEMP_0,Y			; nchstes Byte 0 oder Space?
		tst		TEMP_0
		breq	find_adr_hex_ok
		
		cpi		TEMP_0,' '
		brne	find_adr_err

find_adr_hex_ok:			
		lds		XH,ADR_NIBBLE+0
		swap	XH
		lds		TEMP_0,ADR_NIBBLE+1
		or		XH,TEMP_0

		lds		XL,ADR_NIBBLE+2
		swap	XL
		lds		TEMP_0,ADR_NIBBLE+3
		or		XL,TEMP_0

		rjmp	find_adr_end

find_adr_err:
		sbr		FLAGS,(1<<FLAG_PAR_ERR)

find_adr_end:
		ret

;******************************* Subroutinen *******************************
; Sub: Stoppt den Z80 mit BUSRQ = L
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

z80_stopp:
		ser		TEMP_0
		out		CTRL_DDR,TEMP_0					; Controlport auf Ausgang, alles H!
		cbi		CTRL_PORT,Z80_BUSRQ				; Z80 anhalten
		
z80_stopp_loop:
		dec		TEMP_0
		brne	z80_stopp_loop		

		out		ADR_L_DDR,TEMP_0				; Adressleitungen auf Ausgang, alles H!
		out		ADR_H_DDR,TEMP_0

		ret

;******************************* Subroutinen *******************************
; Sub: Startet den Z80 mit BUSRQ = H
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

z80_start:
		ser		TEMP_0
		clr		TEMP_1
		out		ADR_L_PORT,TEMP_0						; Adressen auf H (PullUp)
		out		ADR_L_DDR,TEMP_1						; und Eingang

		out		ADR_H_PORT,TEMP_0						; Adressen auf H (PullUp)
		out		ADR_H_DDR,TEMP_1						; und Eingang

		ldi		TEMP_0,~(1<<Z80_BUSRQ)					; nur BUSRQ bleibt L
		out		CTRL_PORT,TEMP_0							; 						; 
		out		CTRL_DDR,TEMP_1							; alles Eingang
		sbi		CTRL_PORT,Z80_BUSRQ						; PullUp ein		

		ret

;******************************* Subroutinen *******************************
; Sub: Memory-Dump 1024 Byte ab Adresse als Hex-Liste
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

cmd_dump:
		rcall	find_adr
		sbrc	FLAGS,FLAG_PAR_ERR
		rjmp	cmd_dump_end

		rcall	z80_stopp

		mov		ZH,XH
		mov		ZL,XL
		load_p	Y,1024
		rcall	uart_send_newline
		rcall	uart_send_hex_tab
		rcall	uart_send_newline

		rcall	z80_start

cmd_dump_end:
		ret

;******************************* Subroutinen *******************************
; Sub: Memory-Dump 1024 Byte ab Adresse ber XModem
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

cmd_memory:
		rcall	find_adr
		sbrc	FLAGS,FLAG_PAR_ERR
		rjmp	cmd_memory_end

		load_p	Z,text_xmodem
		rcall	uart_send_rom_text
		rcall	uart_send_newline

		rcall	get_memory

		load_p	X,MEM_BUF
		ldi		TEMP_2,(1024/128)+1				; Anzahl Blcke
		ldi		BLOCK_CNT,1						; erster Block
	
cmd_memory_loop:
		dec		TEMP_2
		breq	cmd_memory_file_end

cmd_memory_loop_wait:
		rcall	uart_receive_byte
		cpi		TEMP_0,ACK
		breq	cmd_memory_block_ok

		cpi		TEMP_0,NAK
		brne	cmd_memory_loop_wait

		dec		BLOCK_CNT						; Block nochmal senden
		rjmp	cmd_memory_block_send

cmd_memory_block_ok:
		dec		TEMP_2
		breq	cmd_memory_file_end		

cmd_memory_block_send:
		rcall	uart_send_xmodem_block
		rjmp	cmd_memory_loop_wait

cmd_memory_file_end:
		ldi		TEMP_0,EOT
		rcall	uart_send_byte
		rjmp	cmd_memory_end
											
cmd_memory_err:
		load_p	Z,text_xmodem_err
		rcall	uart_send_rom_text
		rcall	uart_send_newline

cmd_memory_end:
		ret

;******************************* Subroutinen *******************************
; Sub: liest 1024 Byte vom externen Rom, Adresse in X
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

get_memory:
		rcall	z80_stopp

		load_p	Y,1024
		load_p	Z,MEM_BUF

get_memory_loop:
		out		ADR_H_PORT,XH					; Adresse raus
		out		ADR_L_PORT,XL
		cbi		CTRL_PORT,Z80_MREQ
		cbi		CTRL_PORT,Z80_RD
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		in		TEMP_0,DATA_PIN
		st		Z+,TEMP_0
		adiw	X,1
		sbiw	Y,1
		brne	get_memory_loop

		rcall	z80_stopp

		ret

