;************************** UART Routinen *************************

;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash ber die serielle Schnittstelle
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_rom_text:
		push	ZH
		push	ZL
		push	TEMP_0
		lsl		ZL										; *2 wegen Flash
		rol		ZH	
			
uart_send_rom_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	uart_send_rom_text_end					; Ende

        rcall   uart_send_byte

		rjmp    uart_send_rom_text_loop					; weiter

uart_send_rom_text_end:
		pop		TEMP_0
 		pop		ZL
		pop		ZH

        ret

;******************************* Sub-Debug *********************************
; Sub: Sendet NL/CR ber die serielle Schnittstelle
; Parameter:	-
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_newline:
        push    TEMP_0

		ldi     TEMP_0,$0A
        rcall   uart_send_byte
		ldi     TEMP_0,$0D
        rcall   uart_send_byte

        pop     TEMP_0

		ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als HEX-Dump ber UART
; Parameter: 	TEMP_0
; Return:      	-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte_hex:
        push TEMP_0
		push TEMP_1

        ldi  TEMP_1,7
        push TEMP_0
		swap TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30								; nach Ziffer
		cpi  TEMP_0,$3A								; 0...9
    	brlo wb_1
        add  TEMP_0,TEMP_1                 			; A...F
wb_1:
        rcall uart_send_byte
		pop  TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                   			; nach Ziffer
		cpi  TEMP_0,$3A                   			; 0...9
		brlo wb_2
        add  TEMP_0,TEMP_1                			; A...F
wb_2:
        rcall uart_send_byte

        pop   TEMP_1
		pop   TEMP_0

        ret

;******************************** Sub-Intern *******************************
; Sub: Speicher als 32-Bit (4-Byte) Tabelle ausgeben
; Parameter:    Z - Adresse
; Parameter:    Y - Lnge
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

uart_send_hex_tab:       
		ldi		TEMP_0,4

uart_send_hex_tab_loop:
		lsr     YH                   				; /16
        ror     YL
		dec		TEMP_0
		brne	uart_send_hex_tab_loop

uart_send_hex_tab_line_loop:
        mov		TEMP_0,ZH            				; Adresse senden
        rcall	uart_send_byte_hex
        mov     TEMP_0,ZL
        rcall   uart_send_byte_hex

        ldi     TEMP_1,16							; Bytecounter

uart_send_hex_tab_byte_loop:
        ldi     TEMP_0,' '           				; Leerzeichen
        rcall   uart_send_byte

        ld      TEMP_0,Z+
        rcall   uart_send_byte_hex

        dec		TEMP_1
        brne    uart_send_hex_tab_byte_loop

        rcall   uart_send_newline
        sbiw    Y,1
        brne    uart_send_hex_tab_line_loop

        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:	TEMP_0 Datenbyte
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte:

        sbis    UCSRA,UDRE                  		; Send-Register leer ?
        rjmp    uart_send_byte         	    		; nein

        out     UDR,TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Byte ber UART
; Parameter:    -
; Return:       TEMP_0 Datenbyte wenn FLAG_PROG gesetzt ist
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte:	
		sbis	UCSRA,RXC				        ; warten auf Byte
		rjmp 	uart_receive_byte				; nein

        in		TEMP_0,UDR						; holen

uart_receive_byte_end:
		ret


;******************************* Sub-UART **********************************
; Sub: Empfngt einen String, maximal 16 Zeichen
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

uart_receive_string:
		cbr		FLAGS,(1<<FLAG_STR_OK) | (1<<FLAG_STR_LEN)
		load_p	Z,UART_BUF
		ldi		TEMP_1,16						; Lnge

uart_receive_string_loop:
		rcall	uart_receive_byte
		cpi		TEMP_0,RETURN					; Ende?
		breq	uart_receive_string_ok

		st		Z+,TEMP_0						; speichern
		rcall	uart_send_byte
		dec		TEMP_1
		brne	uart_receive_string_loop		; weiter

		sbr		FLAGS,(1<<FLAG_STR_LEN)	 	; zu viele Zeichen
		rjmp	uart_receive_string_end

uart_receive_string_ok:
		clr		TEMP_0
		st		Z,TEMP_0						; 0-Byte ans Ende
		sbr		FLAGS,(1<<FLAG_STR_OK)			; ok
		rcall	uart_send_newline

uart_receive_string_end:
		ret

;******************************* Sub-UART **********************************
; Sub: Sendet XModem-Block
; Parameter:    BLOCK_CNT - vorige Blocknummer, X - Adresse der Daten
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

uart_send_xmodem_block:
		inc		BLOCK_CNT
		ldi		TEMP_0,SOH
		rcall	uart_send_byte					; Start
		mov		TEMP_0,BLOCK_CNT				; Blocknummer
		rcall	uart_send_byte					; Blocknummer
		com		TEMP_0
		rcall	uart_send_byte					; 1-ner Complement
		
		ldi		BYTE_CNT,128					; Anzahl Bytes
		clr		CHECKSUM
		
cmd_memory_block_loop:
		ld		TEMP_0,X+						; Byte holen
		add		CHECKSUM,TEMP_0					; zur Checksumme dazu
		rcall	uart_send_byte					; Daten
		dec		BYTE_CNT
		brne	cmd_memory_block_loop

		mov		TEMP_0,CHECKSUM
		rcall	uart_send_byte					; Checksumme

		ret
